<?php
include_once (__DIR__.'/s4d.php');
include_once (__DIR__.'/3rdparty/PUC/plugin-update-checker.php');
function definedalilee($accessToken, $s4dKey, $sandbox = true){
    return new s4d\shipping\dalilee([
        'access_key' => $accessToken,
        's4d_license' => $s4dKey,
        'sandbox' => $sandbox,
    ], s4d\shipping\dalilee::secretKey, function ($message) {
        ## DO nothing in this level
    });

}
function dailee_isDokanInstalled(){
    return function_exists('dokan_get_seller_id_by_order');
}
function getdalileeOptions(){
    return get_option('woocommerce_dalilee_settings');
}
function dalilee_shipping_method() {
    if ( ! class_exists( 'dalilee_Shipping_Method' ) ) {
        class dalilee_Shipping_Method extends WC_Shipping_Method {
            /**
             * Constructor for your shipping class
             *
             * @access public
             * @return void
             */
            public $dalilee;
            private $accessToken;
            private $s4dKey;
            private $default_weight = 1;
            private $multiply_by_weight = true;
            private $express_shipping = false;
            private $caching_life;
            private $scheduled_order_trigger;
            private $dalilee_login;
            private $dalilee_password;
            private $dalilee_token;
            private $dalilee_store;
            private $sandbox;
            private $errorMessages = [];
            /**
             * @var mixed
             */

            public function __construct() {
                $this->id                 = 'dalilee';
                $this->method_title       = __( 'Dalilee Shipping', 'dalilee' );
                $this->method_description = __( 'Custom Shipping Method for dalilee', 'dalilee' );
                $this->accessToken = $this->get_option($this->id.'_access_key');
                $this->dalilee_token = $this->get_option('s4d_'.$this->id.'_accessToken');
                $this->s4dKey = $this->get_option($this->id.'_s4d_license');
                $this->default_weight = $this->get_option($this->id.'_default_weight');
                $this->multiply_by_weight = $this->get_option($this->id.'_multiply_by_weight') == 'yes';
                $this->express_shipping = $this->get_option($this->id.'_express_shipping') == 'yes';
                $this->express_shipping = false; ## make sure to ignore users setting
                $this->caching_life = $this->get_option($this->id.'_caching_life');
                $this->scheduled_order_trigger = $this->get_option($this->id.'_scheduled_order_trigger');
                $this->dalilee_login = $this->get_option($this->id.'_email');
                $this->dalilee_password = $this->get_option($this->id.'_password');
                $this->dalilee_store = $this->get_option('s4d_'.$this->id.'_store');
                $this->sandbox = $this->get_option($this->id.'_sandbox') == 'yes';
                $this->dalilee = definedalilee($this->dalilee_token, $this->s4dKey, $this->sandbox);
                $this->dalilee->cacheLifeDays = $this->caching_life;
                // Availability & Countries
                $this->availability = 'including';
                $this->countries = array(
                    'OM'  // Oman
                );

                $this->init();

                $this->enabled = isset( $this->settings['enabled'] ) ? $this->settings['enabled'] : 'yes';
                $this->title = isset( $this->settings['title'] ) ? $this->settings['title'] : __( 'Dalilee Shipping', 'dalilee' );
            }

            /**
             * Init your settings
             *
             * @access public
             * @return void
             */
            function init() {
                // Load the settings API
                $this->init_form_fields();
                $this->init_settings();

                // Save settings in admin if you have any defined
                add_action( 'woocommerce_update_options_shipping_' . $this->id, array( $this, 'process_admin_options' ) );
            }

            /**
             * Define settings field for this shipping
             * @return void
             */
            function init_form_fields() {
                $order_statuses = wc_get_order_statuses();
                $this->form_fields = array(
                    'enabled' => array(
                        'title' => __( 'Enable', 'dalilee' ),
                        'type' => 'checkbox',
                        'description' => __( 'Enable this shipping.', 'dalilee' ),
                        'default' => 'yes'
                    ),
                    'title' => array(
                        'title' => __( 'Title', 'dalilee' ),
                        'type' => 'text',
                        'description' => __( 'Title to be display on site', 'dalilee' ),
                        'default' => __( 'dalilee Shipping', 'dalilee' )
                    ),
                    'weight' => array(
                        'title' => __( 'Weight (kg)', 'dalilee' ),
                        'type' => 'number',
                        'description' => __( 'Maximum allowed weight', 'dalilee' ),
                        'default' => 100
                    ),
                    $this->id.'_default_weight' => array(
                        'title' => __( 'Default Weight (kg)', 'dalilee' ),
                        'type' => 'number',
                        'description' => __( 'In case the weight is not set in a product, will use this value', 'dalilee' ),
                        'default' => 0.5
                    ),
                    $this->id.'_multiply_by_weight' => array(
                        'title' => __( 'Multiply price by weight', 'dalilee' ),
                        'type' => 'checkbox',
                        'description' => __( 'Should multiply shipping price by weight?', 'dalilee' ),
                        'default' => 'no'
                    ),
//                    $this->id.'_express_shipping' => array(
//                        'title' => __( 'Express Shipping', 'dalilee' ),
//                        'type' => 'checkbox',
//                        'description' => __( 'Testing Only: To use Express shipping. This feature is for future and will be not effective now', 'dalilee' ),
//                        'default' => 'no'
//                    ),
                    $this->id.'_caching_life' => array(
                        'title' => __( 'Caching age (Days)', 'dalilee' ),
                        'type' => 'number',
                        'description' => __( 'To avoid calling dalilee API in every request, the plugin will keep a caching file for the number of days specified here. 1 - 3 days is recommended', 'dalilee' ),
                        'default' => 1
                    ),
                    $this->id.'_scheduled_order_trigger' => array(
                        'title' => __( 'Create order on', 'dalilee' ),
                        'type' => 'select',
                        'options' => $order_statuses,
                        'default' => 'wc-processing',
                        'description' => 'API will schedule the order to be delivered automatically when order status change to selected status',
                    ),
//                    $this->id.'_access_key' => array(
//                        'title' => __( 'dalilee Access Token', 'dalilee' ),
//                        'type' => 'text',
//                    ),
                    $this->id.'_email' => array(
                        'title' => __( 'Dalilee Login Email', 'dalilee' ),
                        'type' => 'text',
                    ),
                    $this->id.'_password' => array(
                        'title' => __( 'Dalilee Login Password', 'dalilee' ),
                        'type' => 'text',
                    ),
                    $this->id.'_store' => array(
                        'title' => __( 'Dalilee Store ID', 'dalilee' ),
                        'type' => 'text',
                        'description' => __( 'Example: ST00001', 'dalilee' ),
                    ),
                    $this->id.'_sandbox' => array(
                        'title' => __( 'Use Sandbox', 'dalilee' ),
                        'type' => 'checkbox',
                        'description' => __( 'Use this if you want to use sandbox (testing) environment', 'dalilee' ),
                        'default' => 'yes'
                    ),
                    $this->id.'_dokan_plugin_handling' => array(
                        'title' => __( 'Dokan plugin options', 'dalilee' ),
                        'type' => 'select',
                        'description' => __( 'How to handle shipping with Dokan Multivendor Plugin?', 'dalilee' ),
                        'default' => 2,
                        'options' => [
                            1 => __('Use vendor\'s Access Token and main Access Token as backup', 'dalilee'),
                            2 => __('Use vendor\'s Access Token or disable this shipping method', 'dalilee'),
                            3 => __('Use the main Access Token and disable vendors Access Tokens', 'dalilee'),
                        ]
                    ),
//                    $this->id.'_s4d_license' => array(
//                        'title'       => 'Plugin License Key',
//                        'description' => 'You should get it from <a href="https://www.s4d.om" target="_blank">Source for Development</a>  client area',
//                        'type'        => 'text'
//                    ),
                );
                if(!dailee_isDokanInstalled() || !$this->dalilee->isDokanAddonActive()){
                    unset($this->form_fields[$this->id.'_dokan_plugin_handling']);
                }
                if(isset($_GET['section']) && $_GET['section'] == 'dalilee' && !$this->dalilee->isDokanAddonActive()){
//                    WC_Admin_Settings::add_error( esc_html__( 'Multivendor options are not functional because you need to purchase Dokan addon from client area', 'dalilee' ) );
                }
            }
//            public function validate_s4d_dalilee_accessToken_field( $key, $value ) {
//                $value = trim($value);
//                if(!$this->dalilee->isValidAccessKey($value)){
//                    WC_Admin_Settings::add_error( esc_html__( 'dalilee Access Token is not valid', 'dalilee' ) );
//                    $value = '';
//                }
//                return $value;
//            }
            public function validate_dalilee_email_field( $key, $value ) {
                $value = trim($value);
                if($this->dalilee->isValidLoginCredentials($value, $_POST['woocommerce_dalilee_dalilee_password'])){
                    $this->update_option('s4d_'.$this->id.'_accessToken', $this->dalilee->token);
                }else{
                    WC_Admin_Settings::add_error( esc_html__( 'Invalid login Credentials of dalilee', 'dalilee' ) );
                }
                return $value;
            }
            public function validate_dalilee_store_field( $key, $value ) {
                $value = trim($value);
                if(empty($value) || !$this->dalilee->isValidStoreId($value)){
                    WC_Admin_Settings::add_error( sprintf(esc_html__( 'Invalid store ID %s of dalilee', 'dalilee' ), $value) );
                    $value = '';
                }
                return $value;
            }
            public function validate_dalilee_s4d_license_field( $key, $value ) {
                $value = trim($value);
                $license = $this->dalilee->check_license($value);
                if(strtolower($license['status']) != 'active'){
                    WC_Admin_Settings::add_error( esc_html__( 'S4D License key is not correct for dalilee Plugin', 'dalilee' ) );
                    $value = '';
                }
                return $value;
            }

            /**
             * This function is used to calculate the shipping cost. Within this function we can check for weights, dimensions and other parameters.
             *
             * @access public
             * @param mixed $package
             * @return void
             */
            public function calculate_shipping( $package = array()) {
                $weight = 0;
//                $country = $package["destination"]["country"];
//                echo '<pre dir="ltr">' . print_r($package, true) . '</pre>';
                if(!is_numeric($package['destination']['state'])){
                    $package['destination']['state'] = substr($package['destination']['state'], strlen($package['destination']['country']));
                }
                $stateId = isset($package['destination']['state']) && is_numeric($package['destination']['state']) ? $package['destination']['state'] : 0;
                $cityId = isset($package['destination']['city']) && is_numeric($package['destination']['city']) ? $package['destination']['city'] : 0;
                if($this->dalilee->isDokanAddonActive()) {
                    if (is_array($package['contents']) && count($package['contents']) > 0) {
                        $vendors = [];
                        $vendor_products = [];
                        foreach ($package['contents'] as $product) {
                            $vendor_id = get_post_field('post_author', $product['product_id']);
                            $vendor_data = get_user_meta($vendor_id);
                            $vendor_data['dokan_profile_settings'][0] = unserialize($vendor_data['dokan_profile_settings'][0]);
                            if(!in_array($vendor_id, array_keys($vendor_products))){
                                $vendor_products[$vendor_id] = [];
                            }
                            array_push($vendor_products[$vendor_id], $product['product_id']);
                            if(!empty($vendor_data['dokan_profile_settings'][0]['dalilee_access_token'])){
                                $vendors[$vendor_id] = $vendor_data['dokan_profile_settings'][0]['dalilee_access_token'];
                            }
                        }
                        if(count($vendors) > 0){
                            $deliveryTotal = 0;
                            foreach($vendors as $id => $token){
                                $this->dalilee->config['access_key'] = $token;
                                foreach ($package['contents'] as $item_id => $values) {
                                    if(in_array($values['product_id'], $vendor_products[$id])){
                                        $_product = $values['data'];
                                        $weight = $weight + (empty($_product->get_weight()) || $_product->get_weight() == 0 ? $this->default_weight : $_product->get_weight()) * $values['quantity'];
                                    }
                                }
                                $weight = wc_get_weight($weight, 'kg');
                                $weight = ceil($weight);
                                $dalileeRates = $this->dalilee->getDeliveryPrice($stateId, $cityId, $weight);
                                if ($dalileeRates > 0) {
                                    $rate = array(
                                        'id' => $this->id,
                                        'label' => $this->title,
                                        'cost' => $deliveryTotal,
                                    );
                                    $this->add_rate($rate);
                                }
                            }
                        }
                    }
                }else {
                    // int $stateid, int $regionid, float $weight
                    foreach ($package['contents'] as $item_id => $values) {
                        $_product = $values['data'];
                        $weight = $weight + (empty($_product->get_weight()) || $_product->get_weight() == 0 ? $this->default_weight : $_product->get_weight()) * $values['quantity'];
                    }
                    $weight = wc_get_weight($weight, 'kg');
                    $weight = ceil($weight);
                    $dalileeRates = $this->dalilee->getDeliveryPrice($stateId, $cityId, $weight);
                    if ($dalileeRates > 0) {
                        $rate = array(
                            'id' => $this->id,
                            'label' => $this->title,
                            'cost' => $dalileeRates,
                        );
                        $this->add_rate($rate);
                    }
                }
            }
            public function is_available($data){
                return $this->dalilee->isActiveLicense && (isset($data['destination']['country']) && $data['destination']['country'] == 'OM' || empty($data['destination']['country']));
            }
        }
    }
}

add_action( 'woocommerce_shipping_init', 'dalilee_shipping_method' );

function add_dalilee_shipping_method( $methods ) {
    $methods[] = 'dalilee_Shipping_Method';
    return $methods;
}

add_filter( 'woocommerce_shipping_methods', 'add_dalilee_shipping_method' );

function dalilee_validate_order( $posted )   {
    $packages = WC()->shipping->get_packages();
    $chosen_methods = WC()->session->get( 'chosen_shipping_methods' );
    $dalileeOptions = getdalileeOptions();
    if( is_array( $chosen_methods ) && in_array( 'dalilee', $chosen_methods ) ) {
        $shipping = getdalileeOptions();
        $defaultWeight = $shipping['dalilee_default_weight'];
        $defaultWeight = $defaultWeight == 0? 0.1 : $defaultWeight;
        foreach ( $packages as $i => $package ) {
            if ( $chosen_methods[ $i ] != "dalilee" ) {
                continue;
            }
            $weightLimit = (int) $dalileeOptions['weight'];
            $weight = 0;
            foreach ( $package['contents'] as $item_id => $values ) {
                $_product = $values['data'];
                $productWeight = $_product->get_weight();
                $weight = $weight + (!is_numeric($productWeight)? $defaultWeight : $productWeight) * $values['quantity'];
            }
            $weight = wc_get_weight( $weight, 'kg' );
            if( $weight > $weightLimit ) {
                $message = sprintf( __( 'Sorry, %d kg exceeds the maximum weight of %d kg for %s', 'dalilee' ), $weight, $weightLimit, $dalileeOptions['title'] );
                $messageType = "error";
                if( ! wc_has_notice( $message, $messageType ) ) {
                    wc_add_notice( $message, $messageType );
                }
            }
        }
    }
}
function dalilee_woocommerce_countries( $countries ) {
    $countries = ['OM' => 'Oman'];
    return $countries;
}
function dalilee_woocommerce_states( $states ) {
    $shipping = getdalileeOptions();
    $dalilee = definedalilee($shipping['s4d_dalilee_accessToken']?? '', $shipping['dalilee_s4d_license'], $shipping['dalilee_sandbox'] == 'yes');
    $dalilee->cacheLifeDays = $shipping['dalilee_caching_life'];
    $dalilee->debug = true;
    $states = $dalilee->getStates();
    $states['OM'] = [];
    if(is_array($states) && count($states) > 0){
        foreach($states as $i => $state){
            $state['id'] = $state['id'] ?? '';
            $state['ID'] = $state['id'];
            $states['OM']['OM'.$state['id']] = ($state['name_en']?? '').(isset($state['name_ar']) && !empty($state['name_ar'])? ' - '.$state['name_ar'] : '');
        }
    }
    return $states;
}
function dalilee_billing_fields( $fields, $country ) {
    $fields['billing_city']['type'] = 'city';
    return $fields;
}
function dalilee_shipping_fields( $fields, $country ) {
    $fields['shipping_city']['type'] = 'city';
    return $fields;
}
function dalilee_woocommerce_cities( $field, $key, $args, $value ) {
    $shipping = getdalileeOptions();
    $dalilee = definedalilee($shipping['s4d_dalilee_accessToken']?? '', $shipping['dalilee_s4d_license'], $shipping['dalilee_sandbox'] == 'yes');
    $dalilee->cacheLifeDays = $shipping['dalilee_caching_life'];
    if ( ( ! empty( $args['clear'] ) ) ) {
        $after = '<div class="clear"></div>';
    } else {
        $after = '';
    }
    // Required markup
    if ( $args['required'] ) {
        $args['class'][] = 'validate-required';
        $required = ' <abbr class="required" title="' . esc_attr__( 'required', 'dalilee'  ) . '">*</abbr>';
    } else {
        $required = '';
    }
    // Custom attribute handling
    $custom_attributes = array();
    if ( ! empty( $args['custom_attributes'] ) && is_array( $args['custom_attributes'] ) ) {
        foreach ( $args['custom_attributes'] as $attribute => $attribute_value ) {
            $custom_attributes[] = esc_attr( $attribute ) . '="' . esc_attr( $attribute_value ) . '"';
        }
    }
    // Validate classes
    if ( ! empty( $args['validate'] ) ) {
        foreach( $args['validate'] as $validate ) {
            $args['class'][] = 'validate-' . $validate;
        }
    }
    // field p and label
    $field  = '<p class="form-row ' . esc_attr( implode( ' ', $args['class'] ) ) .'" id="' . esc_attr( $args['id'] ) . '_field">';
    if ( $args['label'] ) {
        $field .= '<label for="' . esc_attr( $args['id'] ) . '" class="' . esc_attr( implode( ' ', $args['label_class'] ) ) .'">' . $args['label']. $required . '</label>';
    }
    // Get Country
    $country_key = $key == 'billing_city' ? 'billing_country' : 'shipping_country';
    $current_cc  = WC()->checkout->get_value( $country_key );
    $state_key = $key == 'billing_city' ? 'billing_state' : 'shipping_state';
    $current_sc  = WC()->checkout->get_value( $state_key );
    // Get country places
    $states = $dalilee->getStates();
    $stateId = str_replace('OM', '', $current_sc);
    $stateId = (int) !empty($stateId)? $stateId : 0;
    $places = $states;
    if ( is_array( $states ) && count($states) > 0 && $stateId > 0 ) {
        $field .= '<select name="' . esc_attr( $key ) . '" id="' . esc_attr( $args['id'] ) . '" class="city_select ' . esc_attr( implode( ' ', $args['input_class'] ) ) .'" ' . implode( ' ', $custom_attributes ) . ' placeholder="' . esc_attr( $args['placeholder'] ) . '">';
        $field .= '<option value="">'. __( 'Select an option&hellip;', 'woocommerce' ) .'</option>';
        $cities = $dalilee->getCities(str_replace('OM', '', $current_sc));
        if(is_array($cities) && count($cities) > 0){
            foreach($cities as $city){
                $field .= '<option value="' . esc_attr( $city['re_id'] ) . '" '.selected( $value, $city['re_id'], false ) . '>' . $city['re_name'] .'</option>';
            }
        }
        $field .= '</select>';
    } else {
        $field .= '<input type="text" class="input-text ' . esc_attr( implode( ' ', $args['input_class'] ) ) .'" value="' . esc_attr( $value ) . '"  placeholder="' . esc_attr( $args['placeholder'] ) . '" name="' . esc_attr( $key ) . '" id="' . esc_attr( $args['id'] ) . '" ' . implode( ' ', $custom_attributes ) . ' />';
    }

    // field description and close wrapper
    if ( $args['description'] ) {
        $field .= '<span class="description">' . esc_attr( $args['description'] ) . '</span>';
    }

    $field .= '</p>' . $after;

    return $field;
}
function dalilee_format_shipments_types($input){
    $output = [];
    if(is_array($input) && count($input) > 0){
        foreach($input as $i => $shipment){
            $output[$shipment['ID']?? ''] = ($shipment['EnglishName']?? '').' - '.$shipment['ArabicName']?? '';
        }
    }
    return $output;
}
function dalilee_woocommerce_product_custom_fields(){
    $shipping = getdalileeOptions();
    $dalilee = definedalilee($shipping['s4d_dalilee_accessToken'], $shipping['dalilee_s4d_license'], $shipping['dalilee_sandbox'] == 'yes');
    $dalilee->cacheLifeDays = $shipping['dalilee_caching_life'];
    $value = get_post_meta( get_the_ID(), 'dalilee_shipment_type', true );
    $args = array(
        'id' => 'dalilee_shipment_type',
        'label' => __('Shipment Type', 'dalilee'),
        'options' => dalilee_format_shipments_types($dalilee->getShipmentsTypes()),
        'value' => !isset($value) || empty($value)? \s4d\shipping\dalilee::defaultShipmentType : $value
    );
    woocommerce_wp_select($args);
}
function dalilee_save_woocommerce_product_custom_fields($post_id){
    $product = wc_get_product($post_id);
    $custom_fields_woocommerce_title = isset($_POST['dalilee_shipment_type']) ? $_POST['dalilee_shipment_type'] : '';
    $product->update_meta_data('dalilee_shipment_type', sanitize_text_field($custom_fields_woocommerce_title));
    $product->save();
}
function dalilee_create_delivery_request($dalilee, $id){
    $order = wc_get_order( $id );
    $parentId = (int) $order->get_parent_id();
    if($parentId > 0){
        dalilee_create_delivery_request($dalilee, $parentId);
    }else {
        $dalileeShipping = getdalileeOptions();
        foreach ($order->get_items('shipping') as $item_id => $item) {
            $shipping_method_id = $item->get_method_id();
        }
        $states = $dalilee->getStates();
        $shippingCountry = $order->get_shipping_country();
        $shippingState = substr($order->get_shipping_state(), strlen($shippingCountry));
        if ($shipping_method_id == 'dalilee') {
            $sub_orders = get_children(
                array('post_parent' => $order->ID, 'post_type' => 'shop_order', 'post_status' => array('wc-pending', 'wc-completed', 'wc-processing', 'wc-on-hold'))
            );
            if (is_array($sub_orders) && count($sub_orders) > 0) {
                $shipmentFee = 0;
                foreach ($sub_orders as $suborder) {
                    $sorder = wc_get_order($suborder->ID);
                    if ($order->get_payment_method() == 'cod') {
                        $dalilee->config['access_key'] = dalilee_token($dalilee, $dalileeShipping, $suborder->ID);
                        $shipmentFee = $dalilee->getDeliveryPrice(isset($shippingState) && is_numeric($shippingState) ? $shippingState : 0, false);
                        $shipmentFee = $shipmentFee[0] ?? 0;
                    }
                    dalilee_create_single_delivery_request($dalilee, $dalileeShipping, $sorder, $suborder->ID, $states, $shippingCountry, $shippingState, $shipmentFee);
                }
            } else {
                dalilee_create_single_delivery_request($dalilee, $dalileeShipping, $order, $id, $states, $shippingCountry, $shippingState);
            }
        }
    }
}
function dalilee_create_single_delivery_request($dalilee, $dalileeShipping, $order, $id, $states, $shippingCountry, $shippingState, $shipmentFee = 0){
    $trackingCode = get_post_meta($id, '_tracking_code');
    $cities = $dalilee->cacheCities();
    $shippingCity = (int) $order->get_shipping_city();
//    echo '<pre dir="ltr">' . print_r([$order->get_shipping_city(), $order], true) . '</pre>'; exit;
    if(!isset($trackingCode[0])){
        $trackingCode[0] = '';
    }
    if(count($trackingCode) == 0 || $trackingCode[0] == '') { // Make sure this item is not already sent to dalilee
        $statesIds = array_column(array_values($states), 'id', $shippingState);
//                    $shipmentType = get_post_meta('product_id');
        $shipmentTypes = [];
        $items = $order->get_items();
        $item_quantity = 0;
        $item_weight = 0;
        foreach($items as $item){
            $shipmentType = get_post_meta($item->get_product_id(), 'dalilee_shipment_type');
            $product        = $item->get_product();
            $item_quantity  += $item->get_quantity();
            $item_weight += $product->get_weight();
            if(!isset($shipmentType[0])){
                $shipmentType = 0;
            }
            if($shipmentType > 0 && !in_array($shipmentType[0], $shipmentTypes)){
                array_push($shipmentTypes, $shipmentType[0]);
            }
        }
        $ThirdPartyReferenceID = rand(1000, 9999).''.$id;
        if ($shippingCountry == 'OM' && in_array($shippingState, $statesIds)) {
            $cityData  = array_filter($cities[$shippingState], function($v, $k) use ($shippingCity){
                return $shippingCity == $v['re_id'];
            }, ARRAY_FILTER_USE_BOTH);
            if(is_array($cityData) && count($cityData) > 0){
                $cityData = reset($cityData);
                if(isset($dalileeShipping['dalilee_dokan_plugin_handling']) && in_array($dalileeShipping['dalilee_dokan_plugin_handling'], [1,2]) && dailee_isDokanInstalled() && function_exists('dokan_get_seller_id_by_order')){
                    $seller_id = dokan_get_seller_id_by_order($id);
                    $storeInfo = dokan_get_store_info($seller_id);
                    switch($dalileeShipping['dalilee_dokan_plugin_handling']){
                        case 1:
                            if(isset($storeInfo['dalilee_access_token']) && !empty($storeInfo['dalilee_access_token'])){
                                $dalilee->config['access_key'] = $storeInfo['dalilee_access_token'];
                            }
                            if(empty($dalilee->config['access_key'])){
                                $dalilee->config['access_key'] = $dalileeShipping['s4d_dalilee_accessToken'];
                            }
                            break;
                        case 2:
                            $dalilee->config['access_key'] = '';
                            if(isset($storeInfo['dalilee_access_token']) && !empty($storeInfo['dalilee_access_token'])){
                                $dalilee->config['access_key'] = $storeInfo['dalilee_access_token'];
                            }
                            break;
                    }
                }
                if(!empty($dalilee->config['access_key'])){
                    $trackingCode = $dalilee->scheduleDelivery([
                        'branchid' => $cityData['branchid'],
                        'quantity' => $item_quantity,
                        'weight' => $item_weight,
                        'trader_code' => $dalileeShipping['dalilee_store'],
                        'region_id' => $order->get_shipping_city(),
                        'order_date' => $order->get_date_created()->date( 'Y-m-d H:i:s' ),
                        'doctype' => 0,
                        'customer_name' =>  $order->get_shipping_first_name().' '.$order->get_shipping_last_name(),
                        'customer_number' =>  $order->get_billing_phone(),
                        'order_price' => $order->is_paid() == 1 && $order->get_payment_method() != 'cod'? 0 : ((double) ($order->get_total() + $shipmentFee)),
                        'wallaya_id' => $cityData['re_walayah'],
                        'zone_id' => $cityData['zone_id']
                    ]);
                    if($trackingCode > 0){
                        update_post_meta($id, '_tracking_code', $trackingCode);
                        update_post_meta($id, '_dalilee_reference_id', $ThirdPartyReferenceID);
                        update_post_meta($id, '_dalilee_shipping_time', date('Y-m-d H:i:s'));
                        $note = sprintf(__('Pickup request has been sent to <b>%s</b> for delivery. Tracking code is <b>%s</b>', 'dalilee'), $dalileeShipping['title'], $trackingCode);
                        $order->add_order_note($note, 1);
                    }else{
                        if(isset($dalilee->errorsList['response'])){
                            $note = sprintf(__('<b>%s</b> has error: %s', 'dalilee'), $dalileeShipping['title'], is_array($dalilee->errorsList['response'])? json_encode($dalilee->errorsList['response']) : $dalilee->errorsList['response']);
                            $order->add_order_note($note);
                        }
                    }
                }else{
                    $note = sprintf(__('<b>%s</b> has error: no Access Token is provided', 'dalilee'), $dalileeShipping['title'], $trackingCode);
                    $order->add_order_note($note);
                }
            }
        }
    }
}
function dalilee_token($dalilee, $dalileeShipping, $id){
    $seller_id = dokan_get_seller_id_by_order($id);
    $storeInfo = dokan_get_store_info($seller_id);
    $output = $dalilee->config['access_key'];
    switch($dalileeShipping['dalilee_dokan_plugin_handling']){
        case 1:
            if(isset($storeInfo['dalilee_access_token']) && !empty($storeInfo['dalilee_access_token'])){
                $output = $storeInfo['dalilee_access_token'];
            }
            if(empty($dalilee->config['access_key'])){
                $output = $dalileeShipping['s4d_dalilee_accessToken'];
            }
            break;
        case 2:
            $output = '';
            if(isset($storeInfo['dalilee_access_token']) && !empty($storeInfo['dalilee_access_token'])){
                $output = $storeInfo['dalilee_access_token'];
            }
            break;
    }
    return $output;
}
function load_dalilee_hooks(){
    $dalileeShipping = getdalileeOptions();
    $dalileeShipping['s4d_dalilee_accessToken'] = $dalileeShipping['s4d_dalilee_accessToken'] ?? '';
    $dalilee = definedalilee($dalileeShipping['s4d_dalilee_accessToken'], $dalileeShipping['dalilee_s4d_license'], $dalileeShipping['dalilee_sandbox'] == 'yes');
    $eventTrigger = $dalileeShipping['dalilee_scheduled_order_trigger'];
    add_action('woocommerce_product_options_shipping', 'dalilee_woocommerce_product_custom_fields');
    add_action('woocommerce_process_product_meta', 'dalilee_save_woocommerce_product_custom_fields');
    add_action( 'woocommerce_review_order_before_cart_contents', 'dalilee_validate_order' , 10 );
    add_action( 'woocommerce_after_checkout_validation', 'dalilee_validate_order' , 10 );
    add_filter( 'woocommerce_states', 'dalilee_woocommerce_states' );
    add_filter( 'woocommerce_billing_fields', 'dalilee_billing_fields', 10, 2 );
    add_filter( 'woocommerce_shipping_fields', 'dalilee_shipping_fields', 10, 2 );
    add_filter( 'woocommerce_form_field_city', 'dalilee_woocommerce_cities', 10, 4 );
    if(!isset($_POST['woocommerce_dalilee_s4d_dalilee_accessToken'])) {
        if (empty($dalileeShipping['dalilee_email'] ?? '')) { WC_Admin_Settings::add_error(esc_html__('dalilee Login Email is not set', 'dalilee')); }
        if (empty($dalileeShipping['dalilee_password'] ?? '')) { WC_Admin_Settings::add_error(esc_html__('dalilee Login Password is not set', 'dalilee'));}
//        if (empty($dalileeShipping['dalilee_s4d_license'] ?? '')) {  WC_Admin_Settings::add_error(esc_html__('S4d License key is not set', 'dalilee')); }
    }
    if(!empty($eventTrigger)){
        add_action('woocommerce_order_status_'.str_replace('wc-', '', $eventTrigger), function($id, $data = []) use ($dalilee, $dalileeShipping){
            dalilee_create_delivery_request($dalilee, $id);
        });
    }
    wp_register_script( "dalilee_script", WP_PLUGIN_URL.'/woocommerce-dalilee-shipping/clientscripts.js', array('jquery') );
    wp_localize_script( 'dalilee_script', 'dalileeAjax', array( 'ajaxurl' => admin_url( 'admin-ajax.php' )));
    wp_localize_script( 'dalilee_script', 'dalilee_cities', array(
        'cities' => ['OM' => $dalilee->cacheCities()],
        'i18n_select_city_text' => esc_attr__( 'Select an option&hellip;', 'dalilee' )
    ) );
    add_filter('woocommerce_formatted_address_replacements', function($c) use ($dalilee){
        $cities = $dalilee->cacheCities();
        $stateId = (int) str_replace('OM', '', $c['{state_code}']);
        if($stateId > 0) {
            $city = $cities[$stateId][array_search($c['{city}'], array_column($cities[$stateId], 're_id'))];
            $c['{city}'] = $city['re_name'];
        }
        return $c;
    });

    wp_enqueue_script( 'jquery' );
    wp_enqueue_script( 'dalilee_script' );
    wp_register_style( 'dalilee_style',  plugin_dir_url(__FILE__).'style.css');
    wp_enqueue_style( 'dalilee_style' );
}
add_action('init', 'load_dalilee_hooks');
function dalilee_extra_field($store_id){
    $profile_info = dokan_get_store_info($store_id);
    $dalileeShipping = getdalileeOptions();
    $dalilee = definedalilee($dalileeShipping['s4d_dalilee_accessToken'], $dalileeShipping['dalilee_s4d_license'], $dalileeShipping['dalilee_sandbox'] == 'yes');
    if($dalilee->isDokanAddonActive()){
        $dalilee_access_token = isset( $profile_info['dalilee_access_token'] ) ? $profile_info['dalilee_access_token'] : '';
        ?>
        <h5><?php _e( 'dalilee Delivery', 'dalilee' ); ?></h5>
        <div class="gregcustom dokan-form-group">
            <label class="dokan-w3 dokan-control-label" for="reg_dalilee_access_token">
                <?php _e( 'Access Token', 'dalilee' ); ?>
            </label>
            <div class="dokan-w5">
                <input type="text" class="dokan-form-control input-md valid" name="dalilee_access_token" id="reg_dalilee_access_token" value="<?php echo $dalilee_access_token; ?>" />
            </div>
        </div>
        <?php
    }
}
function dalilee_extra_field_save($store_id){
    $dokan_settings = dokan_get_store_info($store_id);
    if ( isset( $_POST['dalilee_access_token'] ) && !empty($_POST['dalilee_access_token']) ) {
        $dalileeShipping = getdalileeOptions();
        $dalilee = definedalilee($dalileeShipping['s4d_dalilee_accessToken'], $dalileeShipping['dalilee_s4d_license'], $dalileeShipping['dalilee_sandbox'] == 'yes');
        $dokan_settings['dalilee_access_token'] = trim($_POST['dalilee_access_token']);
        if(!$dalilee->isValidAccessKey($dokan_settings['dalilee_access_token'] )){
            header('Content-Type: application/json');
            $dokan_settings['dalilee_access_token'] = '';
            echo json_encode(['success' => false, 'data' => __('Invalid Access Token of dalilee', 'dalilee')]); exit;
        }
    }
    update_user_meta( $store_id, 'dokan_profile_settings', $dokan_settings );
}
add_filter( 'dokan_settings_form_bottom', 'dalilee_extra_field', 10, 2);
add_action( 'dokan_store_profile_saved', 'dalilee_extra_field_save', 15 );

function dalilee_shipment_control($order){
    $dalileeShipping = getdalileeOptions();
    $id = $order->get_id();
    $sub_orders = get_children(
        array('post_parent' => $id, 'post_type' => 'shop_order', 'post_status' => array('wc-pending', 'wc-completed', 'wc-processing', 'wc-on-hold'))
    );
    $trackingCode = get_post_meta($id, '_tracking_code');
    $trackingCode = is_array($trackingCode) && count($trackingCode) > 0? $trackingCode[0] : 0;
    $nonce = wp_create_nonce("nonce_dalilee_shipment");
    $link = admin_url('admin-ajax.php?action=dalilee_shipment&order_id='.$id.'&nonce='.$nonce.'&action='.($trackingCode == 0? 'create' : 'cancel'));
    $trackingLink = add_query_arg([  'action' => 'dalilee_tracking', 'trackingCode' => $trackingCode, ], 'admin-ajax.php' );
    echo '<h3 style="margin-bottom: 0.8em">'.$dalileeShipping['title'].'</h3>';
    if(is_array($sub_orders) && count($sub_orders) > 0) {
        echo '<span>You need to check suborders page</span>';
    }else{
        if ($trackingCode == 0) {
            // create order
            echo '<a class="dalilee-shipment-trigger button button-secondary" data-nonce="' . $nonce . '" data-order_id="' . $id . '" href="' . $link . '" data-action="create">Schedule delivery with dalilee</a>';
        } else {
            echo '<a class="dalilee-shipment-trigger button button-secondary" data-nonce="' . $nonce . '" data-order_id="' . $id . '" href="' . $link . '" data-action="cancel">Deleted scheduled delivery</a>';
            add_thickbox();
            echo ' <a href="#" class="dalilee-shipment-track button button-secondary" data-tracking="' . $trackingCode . '" data-tracking-link="' . $trackingLink . '">Track Order</a>';
            // delete order
        }
    }
//    echo '<pre dir="ltr">' . print_r($dalilee->orderHistory($trackingCode), true) . '</pre>';
}
add_action( 'woocommerce_admin_order_data_after_shipping_address',  'dalilee_shipment_control');

function dalilee_handle_xhr(){
    header('Content-Type: application/json');
    $dalileeShipping = getdalileeOptions();
    $dalilee = definedalilee($dalileeShipping['s4d_dalilee_accessToken'], $dalileeShipping['dalilee_s4d_license'], $dalileeShipping['dalilee_sandbox'] == 'yes');
    $response = ['status' => 0, 'message' => '', 'tracking' => 0];
    if ( !wp_verify_nonce( $_REQUEST['nonce'], "nonce_dalilee_shipment")) {
        $response['message'] = 'Error with security';
    }
    $order = wc_get_order( $_REQUEST['order_id'] );
    $id = $order->get_id();
    $trackingCode = get_post_meta($id, '_tracking_code');
    $trackingCode = is_array($trackingCode) && count($trackingCode) > 0? $trackingCode[0] : 0;
    $response['tracking'] = (int) $trackingCode;
    if($_REQUEST['btnAction'] == 'cancel' && $trackingCode > 0){
        if($dalilee->isDokanAddonActive()){
            $seller_id = dokan_get_seller_id_by_order($id);
            $storeInfo = dokan_get_store_info($seller_id);
            switch($dalileeShipping['dalilee_dokan_plugin_handling']){
                case 1:
                    if(isset($storeInfo['dalilee_access_token']) && !empty($storeInfo['dalilee_access_token'])){
                        $dalilee->config['access_key'] = $storeInfo['dalilee_access_token'];
                    }
                    if(empty($dalilee->config['access_key'])){
                        $dalilee->config['access_key'] = $dalileeShipping['s4d_dalilee_accessToken'];
                    }
                    break;
                case 2:
                    $dalilee->config['access_key'] = '';
                    if(isset($storeInfo['dalilee_access_token']) && !empty($storeInfo['dalilee_access_token'])){
                        $dalilee->config['access_key'] = $storeInfo['dalilee_access_token'];
                    }
                    break;
            }
        }
        if($dalilee->cancelDelivery($trackingCode)){
            update_post_meta($id, '_tracking_code', '');
            update_post_meta($id, '_dalilee_cancel_shipping_time', date('Y-m-d H:i:s'));
            $note = sprintf(__('Pickup request to <b>%s</b> for delivery with tracking code <b>%s</b> has been canceled', 'dalilee'), $dalileeShipping['title'], $trackingCode);
            $order->add_order_note($note);
        }
    }elseif($trackingCode == 0){
        dalilee_create_delivery_request($dalilee, $id);
    }
    echo json_encode($response); exit;
}
function dalilee_xhr_tracking(){
//    header('Content-Type: application/json');
//    $response = ['status' => (int) 0, 'data' => []];
    $response = '<div style="text-align:center ">No Data</div>';
    $dalileeShipping = getdalileeOptions();
    $dalilee = definedalilee($dalileeShipping['s4d_dalilee_accessToken'], $dalileeShipping['dalilee_s4d_license'], $dalileeShipping['dalilee_sandbox'] == 'yes');
    $trackingCode = (int) $_REQUEST['trackingCode'];
    $order_id = (int) $_REQUEST['order_id'];
    if($trackingCode > 0){
        if($dalilee->isDokanAddonActive()){
            $dalilee->config['access_key'] = dalilee_token($dalilee, $dalileeShipping, $order_id);
        }
        $data = $dalilee->orderHistory($trackingCode);
        if(!empty($data)){
            $response = '<div class="dalilee-tracking-container">'.$data.'</div>';
        }
    }
//    echo json_encode($response); exit;
    exit($response);
}
add_action('wp_ajax_dalilee_shipment', 'dalilee_handle_xhr');
add_action('wp_ajax_nopriv_dalilee_shipment', function(){ echo "You must log in to do this"; die(); });
add_action('wp_ajax_dalilee_tracking', 'dalilee_xhr_tracking');
add_action('wp_ajax_nopriv_dalilee_tracking', function(){ echo "You must log in to do this"; die(); });


function dalilee_orders_list(){
    add_thickbox();
}
function dalilee_track_button($actions, $order){
    $id = $order->get_id();
    $trackingCode = get_post_meta($id, '_tracking_code');
    $trackingCode = is_array($trackingCode) && count($trackingCode) > 0? $trackingCode[0] : 0;
    if($trackingCode > 0) {
        $trackingLink = add_query_arg(['action' => 'dalilee_tracking', 'trackingCode' => $trackingCode, 'order_id' => $id], admin_url('admin-ajax.php'));
        $actions['dalilee_track'] = array(
            'url' => '#' . urlencode($trackingLink),
            'name' => __('Track', 'dalilee'),
        );
    }
    return $actions;
}
add_action('woocommerce_after_account_orders', 'dalilee_orders_list');
add_action('woocommerce_view_order', 'dalilee_orders_list');
add_filter('woocommerce_my_account_my_orders_actions', 'dalilee_track_button', 10, 2);
add_filter('dokan_my_account_my_sub_orders_actions', 'dalilee_track_button', 10, 2);