<?php
namespace s4d\shipping;
if (!class_exists('s4d\payment\shipping', false)) {
    class dalilee {
        const sandboxBaseUrl = 'https://api.dalileeom.com/test/public/api/v1/';
        const productionBaseUrl = 'https://api.dalileeom.com/live/public/api/v1/';
        const secretKey = '2hZ4^P%A4t$1QOu#';
        const webhookTester = 'https://webhook.site/c5b973e8-28ab-4231-89be-3c02ab109517';
        const defaultShipmentType = 11;
        public $config = [];
        public $rsa = [];
        public $connectTimeOut = 30;
        public $payment_id = '';
        public $payment_status = 0;
        public $intentId;
        public $debug = false;
        public $license = '';
        public $s4d_license = '';
        public $licensePath = __DIR__ . '/license.key';
        public $license_secret_key = '';
        public $token;
        public $isActiveLicense = false;
        public $errorFatal;
        public $errorWarning;
        public $responseData = [];
        public $remoteLicenseOnly = false;
        public $headerResponses;
        public $errorsList = [];
        public $warningsList = [];
        public $cacheLifeDays = 1;

        /**
         * thawani constructor.
         * @param array $config
         */
        public function __construct(array $config, $license_secret_key, $errorFatal = '')
        {
            date_default_timezone_set('Asia/Muscat'); ## this is important as dalilee using local time to check validity of invoice
            $this->config = array_merge([
                'access_key' => '',
                's4d_license' => '',
                'remoteLicenseOnly' => false,
                'sandbox' => true,
            ], $config);
            $this->remoteLicenseOnly = $this->config['remoteLicenseOnly'];
            $this->license_secret_key = $license_secret_key;
            $this->errorFatal = $errorFatal;
            foreach ($this->config as $k => $v) {
                if (in_array($k, ['isTestMode', 'remoteLicenseOnly'])) continue;
                if (empty($v)) {
                    array_push($this->errorsList, 'dalilee ' . $k . ' is not set');
                    if (is_callable($this->errorFatal)) {
                        call_user_func_array($this->errorFatal, ['dalilee ' . $k . ' is not set']);
                    } else {
                        trigger_error('dalilee ' . $k . ' is not set', E_USER_ERROR);
                    }
                }
            }
            $this->s4d_license = $config['s4d_license']?? '';
//            $this->handleLicense();
            $this->isActiveLicense = true;
        }

        /**
         * @return bool
         */
        public function is_ssl()
        {
            return isset($_SERVER['HTTPS']) && strtolower($_SERVER['HTTPS']) != 'off' ? true : false;
        }

        /**
         * @return string
         */
        public function currentPageUrl(){
            return urldecode(($this->is_ssl() ? 'https://' : 'http://') . $_SERVER['SERVER_NAME'] . '' . $_SERVER['REQUEST_URI']);
        }

        /**
         * @return false|string
         */
        public function currentPagePlain(){
            return substr($this->currentPageUrl(), 0, strpos($this->currentPageUrl(), '?'));
        }

        /**
         * @param $parameter
         * @return string
         */
        public function getUrl($parameter){
            return ($this->config['sandbox']? self::sandboxBaseUrl : self::productionBaseUrl).$parameter;
        }

        /**
         * @param array $parameters
         * @return array
         */
        public function post(array $parameters, $method = 'POST'){
            $output = ['is_success' => 0, 'response' => ''];
            $parameters = array_merge([
                'url' => '',
                'settings' => [],
                'fields' => [],
                'headers' => [],
                'printHeader' => false,
                'skipSSL' => false,
                'noJSON' => false,
            ], $parameters);
            if (!is_array($parameters['fields'])) $parameters['fields'] = [];
            $fields_string = '';
            $fieldsCount = is_array($parameters['fields'])? count($parameters['fields']) : 0;
            if ($fieldsCount > 0) {
                $fields_string = $parameters['noJSON']? $parameters['fields'] : json_encode($parameters['fields'], JSON_UNESCAPED_UNICODE);
            }
            $parameters['headers'] = array_merge(['content-type' => 'application/json'], $parameters['headers']);
            $headers = array_map(function ($v, $k) {
                return $k . ': ' . $v;
            }, $parameters['headers'], array_keys($parameters['headers']));
            $settings = [
                CURLOPT_URL => $parameters['url'],
                CURLOPT_RETURNTRANSFER => true,
                CURLOPT_FOLLOWLOCATION => true,
                CURLOPT_ENCODING => '',
                CURLOPT_MAXREDIRS => 10,
                CURLOPT_TIMEOUT => $this->connectTimeOut,
                CURLOPT_CONNECTTIMEOUT => $this->connectTimeOut,
                CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
                CURLOPT_CUSTOMREQUEST => $method == 'GET2'? 'GET' : $method,
                CURLOPT_HTTPHEADER => $headers,
            ];
            if ($method == 'GET') {
                if ($fieldsCount > 0) {
                    $settings[CURLOPT_URL] = $parameters['url'].(strpos($parameters['url'], '?') !== false? '&' : '?'). http_build_query($parameters['fields']);
                }
            } else {
                $settings[CURLOPT_POST] = $fieldsCount;
                $settings[CURLOPT_POSTFIELDS] = $fields_string;
            }
            foreach ($parameters['settings'] as $cons => $value) {
                $settings[$cons] = $value;
            }
            $settings[CURLOPT_HEADER] = $parameters['printHeader'] ? 1 : 0;
            $settings[CURLOPT_SSL_VERIFYPEER] = $parameters['skipSSL'] ? 1 : 0;
            $connection = curl_init();
            $this->connection = $connection;
            curl_setopt_array($connection, $settings);
            $content = curl_exec($connection);
            $err = curl_error($connection);
            if ($parameters['printHeader']) {
                preg_match_all('/^Set-Cookie:\s*([^;]*)/mi', $content, $matches);
                $this->headerCookies = array();
                foreach ($matches[1] as $item) {
                    parse_str($item, $cookie);
                    $this->headerCookies = array_merge($this->headerCookies, $cookie);
                }
                $header_len = curl_getinfo($connection, CURLINFO_HEADER_SIZE);
                $this->headerResponses = substr($content, 0, $header_len);
                $content = substr($content, $header_len);
            }
            curl_close($connection);
            if (!$err) {
                $output['is_success'] = 1;
                $output['response'] = $content;
            } else {
                $output['response'] = $err;
            }
            return $output;
        }
        /**
         * @param $directory
         * @param string $type
         */
        public function protect_directory_access($directory, $type = 'access')
        {
            $file = $directory . '/.htaccess';
            if (!is_dir($directory)) {
                $directory = '../' . $directory;
            }
            if (!file_exists($file)):
                $file = '../' . $file;
            endif;
            if (!file_exists($file)):
                $file = is_dir($directory) ? $directory . '/.htaccess' : '../' . $directory . '/.htaccess';
                $handle = fopen($file, 'w');
                fwrite($handle, ($type == 'comprehensive' ? 'deny from all' : 'Options -Indexes'));
                fclose($handle);
            endif;
        }

        public function check_license($licensekey, $localkey = '')
        {
            // Enter the url to your WHMCS installation here
            $whmcsurl = 'https://www.s4d.om/hosting/';
            // Must match what is specified in the MD5 Hash Verification field
            // of the licensing product that will be used with this check.
            $licensing_secret_key = $this->license_secret_key;
            // The number of days to wait between performing remote license checks
            $localkeydays = 15;
            // The number of days to allow failover for after local key expiry
            $allowcheckfaildays = 10;

            // -----------------------------------
            //  -- Do not edit below this line --
            // -----------------------------------

            $check_token = time() . md5(mt_rand(100000000, mt_getrandmax()) . $licensekey);
            $checkdate = date("Ymd");
            $domain = $_SERVER['SERVER_NAME'];
            $usersip = isset($_SERVER['SERVER_ADDR']) ? $_SERVER['SERVER_ADDR'] : $_SERVER['LOCAL_ADDR'];
            $dirpath = dirname(__FILE__);
            $verifyfilepath = 'modules/servers/licensing/verify.php';
            $localkeyvalid = false;
            if ($localkey) {
                $localkey = str_replace("\n", '', $localkey); # Remove the line breaks
                $localdata = substr($localkey, 0, strlen($localkey) - 32); # Extract License Data
                $md5hash = substr($localkey, strlen($localkey) - 32); # Extract MD5 Hash
                if ($md5hash == md5($localdata . $licensing_secret_key)) {
                    $localdata = strrev($localdata); # Reverse the string
                    $md5hash = substr($localdata, 0, 32); # Extract MD5 Hash
                    $localdata = substr($localdata, 32); # Extract License Data
                    $localdata = base64_decode($localdata);
                    $localkeyresults = json_decode($localdata, true);
                    $originalcheckdate = $localkeyresults['checkdate'];
                    if ($md5hash == md5($originalcheckdate . $licensing_secret_key)) {
                        $localexpiry = date("Ymd", mktime(0, 0, 0, date("m"), date("d") - $localkeydays, date("Y")));
                        if ($originalcheckdate > $localexpiry) {
                            $localkeyvalid = true;
                            $results = $localkeyresults;
                            $results['validdomain'] = !isset($results['validdomain']) ? '' : $results['validdomain'];
                            $validdomains = explode(',', $results['validdomain']);
                            if (!in_array($_SERVER['SERVER_NAME'], $validdomains)) {
                                $localkeyvalid = false;
                                $localkeyresults['status'] = "Invalid";
                                $results = array();
                            }
                            $results['validip'] = !isset($results['validip']) ? '' : $results['validip'];
                            $validips = explode(',', $results['validip']);
                            if (!in_array($usersip, $validips)) {
                                $localkeyvalid = false;
                                $localkeyresults['status'] = "Invalid";
                                $results = array();
                            }
                            $results['validdirectory'] = !isset($results['validdirectory']) ? '' : $results['validdirectory'];
                            $validdirs = explode(',', $results['validdirectory']);
                            if (!in_array($dirpath, $validdirs)) {
                                $localkeyvalid = false;
                                $localkeyresults['status'] = "Invalid";
                                $results = array();
                            }
                            $results['addons'] = $this->addonParser($results['addons'] ?? '');
                        }
                    }
                }
            }
            if (!$localkeyvalid) {
                $responseCode = 0;
                $postfields = array(
                    'licensekey' => $licensekey,
                    'domain' => $domain,
                    'ip' => $usersip,
                    'dir' => $dirpath,
                );
                if ($check_token) $postfields['check_token'] = $check_token;
                $query_string = '';
                foreach ($postfields as $k => $v) {
                    $query_string .= $k . '=' . urlencode($v) . '&';
                }
                if (function_exists('curl_exec')) {
                    $ch = curl_init();
                    curl_setopt($ch, CURLOPT_URL, $whmcsurl . $verifyfilepath);
                    curl_setopt($ch, CURLOPT_POST, 1);
                    curl_setopt($ch, CURLOPT_POSTFIELDS, $query_string);
                    curl_setopt($ch, CURLOPT_TIMEOUT, 30);
                    curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
                    $data = curl_exec($ch);
                    $responseCode = curl_getinfo($ch, CURLINFO_HTTP_CODE);
                    curl_close($ch);
                } else {
                    $responseCodePattern = '/^HTTP\/\d+\.\d+\s+(\d+)/';
                    $fp = @fsockopen($whmcsurl, 80, $errno, $errstr, 5);
                    if ($fp) {
                        $newlinefeed = "\r\n";
                        $header = "POST " . $whmcsurl . $verifyfilepath . " HTTP/1.0" . $newlinefeed;
                        $header .= "Host: " . $whmcsurl . $newlinefeed;
                        $header .= "Content-type: application/x-www-form-urlencoded" . $newlinefeed;
                        $header .= "Content-length: " . @strlen($query_string) . $newlinefeed;
                        $header .= "Connection: close" . $newlinefeed . $newlinefeed;
                        $header .= $query_string;
                        $data = $line = '';
                        @stream_set_timeout($fp, 20);
                        @fputs($fp, $header);
                        $status = @socket_get_status($fp);
                        while (!@feof($fp) && $status) {
                            $line = @fgets($fp, 1024);
                            $patternMatches = array();
                            if (!$responseCode
                                && preg_match($responseCodePattern, trim($line), $patternMatches)
                            ) {
                                $responseCode = (empty($patternMatches[1])) ? 0 : $patternMatches[1];
                            }
                            $data .= $line;
                            $status = @socket_get_status($fp);
                        }
                        @fclose($fp);
                    }
                }
                if ($responseCode != 200) {
                    $localexpiry = date("Ymd", mktime(0, 0, 0, date("m"), date("d") - ($localkeydays + $allowcheckfaildays), date("Y")));
                    if ($originalcheckdate > $localexpiry) {
                        $results = $localkeyresults;
                    } else {
                        $results = array();
                        $results['status'] = "Invalid";
                        $results['description'] = "Remote Check Failed";
                        return $results;
                    }
                } else {
                    preg_match_all('/<(.*?)>([^<]+)<\/\\1>/i', $data, $matches);
                    $results = array();
                    foreach ($matches[1] as $k => $v) {
                        $results[$v] = $matches[2][$k];
                    }
                }
                if (!is_array($results)) {
                    die("Invalid License Server Response");
                }
                if (isset($results['md5hash']) && $results['md5hash']) {
                    if ($results['md5hash'] != md5($licensing_secret_key . $check_token)) {
                        $results['status'] = "Invalid";
                        $results['description'] = "MD5 Checksum Verification Failed";
                        return $results;
                    }
                }
                if ($results['status'] == "Active") {
                    $results['checkdate'] = $checkdate;
                    $data_encoded = json_encode($results);
                    $data_encoded = base64_encode($data_encoded);
                    $data_encoded = md5($checkdate . $licensing_secret_key) . $data_encoded;
                    $data_encoded = strrev($data_encoded);
                    $data_encoded = $data_encoded . md5($data_encoded . $licensing_secret_key);
                    $data_encoded = wordwrap($data_encoded, 80, "\n", true);
                    $results['localkey'] = $data_encoded;
                }
                $results['addons'] = $this->addonParser($results['addons'] ?? '');
                $results['remotecheck'] = true;
            }
            unset($postfields, $data, $matches, $whmcsurl, $licensing_secret_key, $checkdate, $usersip, $localkeydays, $allowcheckfaildays, $md5hash);
            return $results;
        }
        public function addonParser($input){
            $output = [];
            $input = explode(';', $input);
            $input = !is_array($input)? [] : $input;
            if(count($input) > 0){
                $key = '';
                foreach($input as $item){
                    $item = explode('=', $item);
                    if( $item[0] == 'name'){
                        $key = stripos($item[1], 'dokan') !== false? 'dokan' : $item[1];
                    }
                    $output[$key][$item[0]] = $item[1] ?? '';
                }
            }
            return $output;
        }
        public function isDokanAddonActive(){
            return isset($this->license['addons']) && isset($this->license['addons']['dokan']) && strtolower($this->license['addons']['dokan']['status']) == 'active';
        }

        public function keyFile($key = '', $isWrite = 0)
        {
            $output = $key;
            try {
                if ($isWrite == 1 || (file_exists($this->licensePath) && $isWrite == 0)) {
                    $handle = fopen($this->licensePath, $isWrite == 1 ? 'w' : 'r');
                    if ($handle) {
                        if ($isWrite == 1) {
                            fwrite($handle, $key);
                        } else {
                            $output = fread($handle, filesize($this->licensePath));
                        }
                        fclose($handle);
                    } else {
                        throw new \Exception('Error');
                    }
                }
            } catch (\Exception $e) {
                die($e->getTraceAsString());
            }
            return $output;
        }

        public function handleLicense()
        {
            if ($this->remoteLicenseOnly) {
                $this->license = $this->check_license($this->s4d_license);
            } else {
                $this->license = $this->check_license($this->s4d_license, $this->keyFile());
            }
            if ($this->license['status'] != 'Active') {
                $message = 'Cannot verify class license, please contact s4d.om';
                if (is_callable($this->errorFatal)) {
                    call_user_func_array($this->errorFatal, [$message]);
                } else {
                    die($message);
                }
            } else {
                if (isset($this->license['localkey']) && !empty($this->license['localkey']) && !$this->remoteLicenseOnly) {
                    $this->keyFile($this->license['localkey'], 1);
                }
                $this->isActiveLicense = true;
            }
        }

        /**
         * @param $input
         * @param int $return
         * @param string $cssClass
         * @return string
         */
        public function iprint_r($input, $return = 0, $cssClass = '')
        {
            $output = '<pre dir="ltr"' . (!empty($cssClass) ? ' class="' . $cssClass . '"' : '') . '>' . (is_array($input) || is_object($input) ? print_r($input, 1) : $input) . '</pre>';
            array_push($this->warningsList, $output);
            if (isset($this->errorWarning) && is_callable($this->errorWarning)) {
                call_user_func_array($this->errorWarning, [$output]);
            } else {
                if ($return != 0) return $output; else echo $output;
            }
        }
        public function isValidAccessKey($input){
            $output = false;
            $server = $this->post([
                'url' => $this->getUrl('States'),
                'headers' => [
                    'Authorization' => 'Bearer '.$input,
                    'Content-Type' => 'application/json'
                ]
            ], 'GET');
            if ($server['is_success'] == 1 && !empty($server['response'])) {
                $data = json_decode($server['response'], true);
                $output = is_array($data) && count($data) > 0;
            }
            return $output;
        }
        public function isValidLoginCredentials($email, $password){
            $output = false;
            $server = $this->post([
                'url' => $this->getUrl('login'),
                'headers' => [
                    'Content-Type' => 'application/json'
                ],
                'fields' => [
                    'email' => $email,
                    'password' => $password,
                    'password_confirmation' => $password
                ]
            ], 'POST');
            if ($server['is_success'] == 1 && !empty($server['response'])) {
                $data = json_decode($server['response'], true);
                if(is_array($data) && count($data) > 0){
                    if(isset($data['token']) && !empty($data['token'])){
                        $this->token = $data['token'];
                        $output = true;
                    }
                }
            }
            return $output;
        }
        public function isValidStoreId($store_id){
            $output = false;
            $store_id = strtoupper($store_id);
            $server = $this->post([
                'url' => $this->getUrl('verifyStore'),
                'headers' => [
                    'Authorization' => 'Bearer '.$this->config['access_key'],
                    'Content-Type' => 'application/json'
                ],
                'fields' => [
                    'trader_code' => $store_id
                ]
            ], 'POST');
            if ($server['is_success'] == 1 && !empty($server['response'])) {
                $data = json_decode($server['response'], true);
                if(is_array($data) && count($data) > 0 && is_array($data['data'] ?? '') && count($data['data']) > 0){
                    if(isset($data['data']['store_code']) && !empty($data['data']['store_code']) && strtoupper($data['data']['store_code']) == $store_id){
                        $output = true;
                    }
                }
            }
            return $output;
        }
        public function getStates(){
            $fileCache = __DIR__.'/statesList.json';
            $output = $this->jsonCache($fileCache);
            if(count($output) == 0) {
                $server = $this->post([
                    'url' => $this->getUrl('wilayalist'),
                    'headers' => [
                        'Authorization' => 'Bearer '.$this->config['access_key'],
                        'Content-Type' => 'application/json'
                    ]
                ], 'GET');
                if ($server['is_success'] == 1 && !empty($server['response'])) {
                    $output = json_decode($server['response'], true);
                    $output = isset($output['data'])? $output['data'] : [];
                    $this->jsonCache($fileCache, $output);
                }
            }
            return $output;
        }
        public function getCities($id){
            $server = $this->post([
                'url' => $this->getUrl('regions'),
                'headers' => [
                    'Authorization' => 'Bearer '.$this->config['access_key'],
                    'Content-Type' => 'application/json'
                ],
                'fields' => [
                    'wilaya_id' => $id
                ]
            ], 'POST');
            if ($server['is_success'] == 1 && !empty($server['response'])) {
                $output = json_decode($server['response'], true);
                $output = isset($output['data']) ? $output['data'] : [];
            }
            return $output;
        }
        public function cacheCities($print = false){ ## For Later stage
            ini_set('memory_limit', -1);
            $fileCache = __DIR__.'/citiesList.json';
            $output = $this->jsonCache($fileCache);
            if(count($output) == 0) {
                $output = [];
                $states = $this->getStates();
                foreach($states as $row) {
                    if($print){ ob_start(); }
                    $output[$row['id']] = [];
                    $server = $this->post([
                        'url' => $this->getUrl('regions'),
                        'headers' => [
                            'Authorization' => 'Bearer '.$this->config['access_key'],
                            'Content-Type' => 'application/json'
                        ],
                        'fields' => [
                            'wilaya_id' => $row['id']
                        ]
                    ], 'POST');
                    if ($server['is_success'] == 1 && !empty($server['response'])) {
                        $output[$row['id']] = json_decode($server['response'], true);
                        $output[$row['id']] = isset($output[$row['id']]['data']) ? $output[$row['id']]['data'] : [];
                    }
                    if($print){
                        echo "\33[1;32msuccess:\33[m".$row['id'].', '.$row['name_en']."\n"; flush(); ob_flush(); usleep(50000);
                        ob_end_clean();
                    }
                }
                $this->jsonCache($fileCache, $output);
            }
            return $output;
        }
        public function getDeliveryPrice(int $stateid, int $regionid, float $weight){
            $output = [];
            $server = $this->post([
                'url' => $this->getUrl('calculatePrice'),
                'headers' => [
                    'Authorization' => 'Bearer '.$this->config['access_key'],
                    'content-type' => 'multipart/form-data'
                ],
                'fields' => [
                    'willaya' => $stateid,
                    'region' => $regionid,
                    'weight' => $weight,
                ],
                'noJSON' => true,
            ], 'POST');
            if($server['is_success'] == 1 && !empty($server['response'])){
                $output = json_decode($server['response'], true);
                $output = !is_array($output) && !empty($output)? [$output] : $output;
                $output = is_array($output['data'] ?? []) && isset($output['data']['price_val'])? $output['data']['price_val'] : 0;
            }
            return $output;
        }
        public function getShipmentsTypes(){
            $fileCache = __DIR__.'/shipmentsTypes.json';
            $output = $this->jsonCache($fileCache);
            if(count($output) == 0){
                $server = $this->post([
                    'url' => $this->getUrl('ShipmentsTypes'),
                    'headers' => [
                        'Authorization' => 'Bearer '.$this->config['access_key'],
                        'Content-Type' => 'application/json'
                    ]
                ], 'GET');
                if($server['is_success'] == 1 && !empty($server['response'])){
                    $output = json_decode($server['response'], true);
                    $this->jsonCache($fileCache, $output);
                }
            }
            return $output;
        }
        public function scheduleDelivery($parameters = []){
            $output = (int) 0;
            $parameters = array_merge([
                'branchid' => '',
                'quantity' => '',
                'weight' => '',
                'trader_code' => '',
                'region_id' => '',
                'order_date' => 0,
                'doctype' => '',
                'customer_name' => '',
                'customer_number' => '',
                'order_price' => '',
                'wallaya_id' => '',
                'zone_id' => ''
            ], $parameters);
            $server = $this->post([
                'url' => $this->getUrl('addorder'),
                'headers' => [
                    'Authorization' => 'Bearer '.$this->config['access_key'],
                    'Content-Type' => 'application/json'
                ],
                'fields' => $parameters,
            ], 'POST');
            if($server['is_success'] == 1 && !empty($server['response'])){
                $server['response'] = json_decode($server['response'], true);
                $output = (int) isset($server['response']['data'][0]['orderid']) && is_numeric($server['response']['data'][0]['orderid']) && ((int) $server['response']['data'][0]['orderid']) > 0? $server['response']['data'][0]['orderid'] : 0;
                if($output == 0){
                    $this->errorsList = $server;
                }
            }
            return $output;
        }
        public function cancelDelivery($input){
            $status = false;
            $server = $this->post([
                'url' => $this->getUrl('CanceledOrder'),
                'headers' => [
                    'Authorization' => 'Bearer '.$this->config['access_key'],
                    'Content-Type' => 'application/json'
                ],
                'fields' => [
                    'TrackingCode' => (int) $input
                ],
            ], 'POST');
            if($server['is_success'] == 1 && !empty($server['response'])){
                $status = strpos(strtolower($server['response']), 'canceled successfully') !== false;
            }
            return $status;
        }
        public function orderHistory($input){
            $output = '<div style="text-align:center ">No Data</div>';
            $server = $this->post([
                'url' => $this->getUrl('getOrderLog'),
                'headers' => [
                    'Authorization' => 'Bearer '.$this->config['access_key'],
                    'content-type' => 'multipart/form-data'
                ],
                'noJSON' => true,
                'fields' => [
                    'order_no' => 'A'.((int) $input)
                ]
            ], 'POST');
            if($server['is_success'] == 1 && !empty($server['response'])){
                $data = json_decode($server['response'], true);
                if(is_array($data) && count($data) > 0 && !empty($data['data'])){
                    $output = $data['data'];
                }
            }
            return $output;
        }
        public function jsonCache($file, $input = []){
            $data = ['create_date' => date('Y-m-d H:i'), 'data' => $input];
            $output = [];
            $this->cacheLifeDays = !isset($this->cacheLifeDays) || empty($this->cacheLifeDays)  || !is_numeric($this->cacheLifeDays) || $this->cacheLifeDays == 0? 1 : $this->cacheLifeDays;
            if(file_exists($file)){
                $handle = fopen($file, 'r');
                $content = json_decode(fread($handle, filesize($file)), true);
                fclose($handle);
                if(is_array($content) && isset($content['create_date']) && !empty($content['create_date'])){
                    $fileDate = new \DateTime($content['create_date']);
                    $currentDate = new \DateTime();
                    $interval = $fileDate->diff($currentDate);
                    $output = $content['data'];
                    if($interval->format('%a') > $this->cacheLifeDays && !in_array($file, [__DIR__.'/citiesList.json'])){
                        unlink($file);
                    }
                }else{
                    unlink($file);
                }
            }
            if(is_array($input) && count($input) > 0){
                $handle = fopen($file, 'w');
                fwrite($handle, json_encode($data));
                fclose($handle);
            }
            return $output;
        }
    }
}